/**************************************************************************************************

Parsing.h -- (C) Niklas Een, Niklas Srensson, 2004
Extended by Antonin Lejsek 2005

Parses input, builds and register constrs

**************************************************************************************************/


#include "parsing.h"


Lit  parseToLit(int i) { Lit p; p.x = ((i>0)? i+i-2: abs(i+i)-1); return p; };
//=================================================================================================
// DIMACS Parser:


static void skipWhitespace(char*& in) {
    while ((*in >= 9 && *in <= 13) || *in == 32)
        in++; }

static void skipLine(char*& in) {
    for (;;){
        if (*in == 0) return;
        if (*in == '\n') { in++; return; }
        in++; } }

static int parseInt(char*& in) {
    int     val = 0;
    bool    neg = false;
    skipWhitespace(in);
    if      (*in == '-') neg = true, in++;
    else if (*in == '+') in++;
    if (*in < '0' || *in > '9') fprintf(stderr, "PARSE ERROR! Unexpected char: %c\n", *in), exit(1);
    while (*in >= '0' && *in <= '9')
        val = val*10 + (*in - '0'),
        in++;
    return neg ? -val : val; }



void Constr_reg::readVector(char*& in, Solver& S, vec<int>& ints){
	int     parsed_int;
    ints.clear();
    for (;;){
        parsed_int = parseInt(in);
        if (parsed_int == 0) break;
		ints.push(parsed_int);        
    }
}



void Constr_reg::registerConstrToSAT(Solver& S,vec<int>& ints,const char*type){
	if (S.ok){ 
		Constr* c; 
		for(int i=0;i<constrs.size();++i){
			if(!strcmp(constrs[i]->getType(),type)){
				S.ok = constrs[i]->constr_new(S, ints, c); 
				if (c != NULL) S.constrs.push(c); 
				break;
			}		
		}
	}
}

void Constr_reg::registerConstr(Constr* c){
	for(int i=0;i<constrs.size();++i)
		if(constrs[i]==c)return;
	constrs.push(c);
}

//=================================================================================================
//=================================================================================================
// Helpers:


// Reads an input stream to end-of-file and returns the result as a 'char*' terminated by '\0'
// (dynamic allocation in case 'in' is standard input).
//
char* readFile(FILE* in, int& out_size = *(int*)NULL)
{
    char*   data = xmalloc<char>(65536);
    int     cap  = 65536;
    int     size = 0;

    while (!feof(in)){
        if (size == cap){
            cap *= 2;
            data = xrealloc(data, cap); }
        size += fread(&data[size], 1, 65536, in);
    }
    data = xrealloc(data, size+1);
    data[size] = '\0';

    if (&out_size != NULL)
        out_size = size;

    return data;
}



bool Constr_reg::parse_DIMACS_main(char* in, Solver& S) {
    vec<int>    ints;
	char*type;
    for (;;){
        skipWhitespace(in);
        if (*in == 0) break;
		if (*in == 'c' || *in == 'p'){
            skipLine(in);
			continue;
		}
		switch(*in){
			case 'x':
				type=in+1;
				while(*in>='a' && *in<='z' ||
					  *in>='A' && *in<='Z' ||
					  *in>='0' && *in<='9')++in;
				*in=0;
				++in;
				break;
			default:
			type="clause";
		}
        
		readVector(in,S,ints);
		registerConstrToSAT(S,ints,type);            
        if (!S.okay())
            return false;
        
    }
    S.simplifyDB();
    return S.okay();
}

bool Constr_reg::parse_DIMACS(FILE* in, Solver& S) {
    char* text = readFile(in);
    bool ret = parse_DIMACS_main(text, S);
    free(text);
    return ret; }